/*
 * Estonian ID card plugin for web browsers
 *
 * Copyright (C) 2010-2011 Codeborne <info@codeborne.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef __PKCS11_ERRORS_H__
#define __PKCS11_ERRORS_H__

#define CRYPTOKI_COMPAT
#include "pkcs11.h"

typedef struct {
	CK_RV result;
	const char *message;
} pkcs11_error_mapping;

static const pkcs11_error_mapping pkcs11_error_map[] = {
{CKR_OK, "CKR_OK"},
{CKR_CANCEL, "CKR_CANCEL"},
{CKR_HOST_MEMORY, "CKR_HOST_MEMORY"},
{CKR_SLOT_ID_INVALID, "CKR_SLOT_ID_INVALID"},
{CKR_GENERAL_ERROR, "CKR_GENERAL_ERROR"},
{CKR_FUNCTION_FAILED, "CKR_FUNCTION_FAILED"},
{CKR_ARGUMENTS_BAD, "CKR_ARGUMENTS_BAD"},
{CKR_NO_EVENT, "CKR_NO_EVENT"},
{CKR_NEED_TO_CREATE_THREADS, "CKR_NEED_TO_CREATE_THREADS"},
{CKR_CANT_LOCK, "CKR_CANT_LOCK"},
{CKR_ATTRIBUTE_READ_ONLY, "CKR_ATTRIBUTE_READ_ONLY"},
{CKR_ATTRIBUTE_SENSITIVE, "CKR_ATTRIBUTE_SENSITIVE"},
{CKR_ATTRIBUTE_TYPE_INVALID, "CKR_ATTRIBUTE_TYPE_INVALID"},
{CKR_ATTRIBUTE_VALUE_INVALID, "CKR_ATTRIBUTE_VALUE_INVALID"},
{CKR_DATA_INVALID, "CKR_DATA_INVALID"},
{CKR_DATA_LEN_RANGE, "CKR_DATA_LEN_RANGE"},
{CKR_DEVICE_ERROR, "CKR_DEVICE_ERROR"},
{CKR_DEVICE_MEMORY, "CKR_DEVICE_MEMORY"},
{CKR_DEVICE_REMOVED, "CKR_DEVICE_REMOVED"},
{CKR_ENCRYPTED_DATA_INVALID, "CKR_ENCRYPTED_DATA_INVALID"},
{CKR_ENCRYPTED_DATA_LEN_RANGE, "CKR_ENCRYPTED_DATA_LEN_RANGE"},
{CKR_FUNCTION_CANCELED, "CKR_FUNCTION_CANCELED"},
{CKR_FUNCTION_NOT_PARALLEL, "CKR_FUNCTION_NOT_PARALLEL"},
{CKR_FUNCTION_NOT_SUPPORTED, "CKR_FUNCTION_NOT_SUPPORTED"},
{CKR_KEY_HANDLE_INVALID, "CKR_KEY_HANDLE_INVALID"},
{CKR_KEY_SIZE_RANGE, "CKR_KEY_SIZE_RANGE"},
{CKR_KEY_TYPE_INCONSISTENT, "CKR_KEY_TYPE_INCONSISTENT"},
{CKR_KEY_NOT_NEEDED, "CKR_KEY_NOT_NEEDED"},
{CKR_KEY_CHANGED, "CKR_KEY_CHANGED"},
{CKR_KEY_NEEDED, "CKR_KEY_NEEDED"},
{CKR_KEY_INDIGESTIBLE, "CKR_KEY_INDIGESTIBLE"},
{CKR_KEY_FUNCTION_NOT_PERMITTED, "CKR_KEY_FUNCTION_NOT_PERMITTED"},
{CKR_KEY_NOT_WRAPPABLE, "CKR_KEY_NOT_WRAPPABLE"},
{CKR_KEY_UNEXTRACTABLE, "CKR_KEY_UNEXTRACTABLE"},
{CKR_MECHANISM_INVALID, "CKR_MECHANISM_INVALID"},
{CKR_MECHANISM_PARAM_INVALID, "CKR_MECHANISM_PARAM_INVALID"},
{CKR_OBJECT_HANDLE_INVALID, "CKR_OBJECT_HANDLE_INVALID"},
{CKR_OPERATION_ACTIVE, "CKR_OPERATION_ACTIVE"},
{CKR_OPERATION_NOT_INITIALIZED, "CKR_OPERATION_NOT_INITIALIZED"},
{CKR_PIN_INCORRECT, "CKR_PIN_INCORRECT"},
{CKR_PIN_INVALID, "CKR_PIN_INVALID"},
{CKR_PIN_LEN_RANGE, "CKR_PIN_LEN_RANGE"},
{CKR_PIN_EXPIRED, "CKR_PIN_EXPIRED"},
{CKR_PIN_LOCKED, "CKR_PIN_LOCKED"},
{CKR_SESSION_CLOSED, "CKR_SESSION_CLOSED"},
{CKR_SESSION_COUNT, "CKR_SESSION_COUNT"},
{CKR_SESSION_HANDLE_INVALID, "CKR_SESSION_HANDLE_INVALID"},
{CKR_SESSION_PARALLEL_NOT_SUPPORTED, "CKR_SESSION_PARALLEL_NOT_SUPPORTED"},
{CKR_SESSION_READ_ONLY, "CKR_SESSION_READ_ONLY"},
{CKR_SESSION_EXISTS, "CKR_SESSION_EXISTS"},
{CKR_SESSION_READ_ONLY_EXISTS, "CKR_SESSION_READ_ONLY_EXISTS"},
{CKR_SESSION_READ_WRITE_SO_EXISTS, "CKR_SESSION_READ_WRITE_SO_EXISTS"},
{CKR_SIGNATURE_INVALID, "CKR_SIGNATURE_INVALID"},
{CKR_SIGNATURE_LEN_RANGE, "CKR_SIGNATURE_LEN_RANGE"},
{CKR_TEMPLATE_INCOMPLETE, "CKR_TEMPLATE_INCOMPLETE"},
{CKR_TEMPLATE_INCONSISTENT, "CKR_TEMPLATE_INCONSISTENT"},
{CKR_TOKEN_NOT_PRESENT, "CKR_TOKEN_NOT_PRESENT"},
{CKR_TOKEN_NOT_RECOGNIZED, "CKR_TOKEN_NOT_RECOGNIZED"},
{CKR_TOKEN_WRITE_PROTECTED, "CKR_TOKEN_WRITE_PROTECTED"},
{CKR_UNWRAPPING_KEY_HANDLE_INVALID, "CKR_UNWRAPPING_KEY_HANDLE_INVALID"},
{CKR_UNWRAPPING_KEY_SIZE_RANGE, "CKR_UNWRAPPING_KEY_SIZE_RANGE"},
{CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT, "CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT"},
{CKR_USER_ALREADY_LOGGED_IN, "CKR_USER_ALREADY_LOGGED_IN"},
{CKR_USER_NOT_LOGGED_IN, "CKR_USER_NOT_LOGGED_IN"},
{CKR_USER_PIN_NOT_INITIALIZED, "CKR_USER_PIN_NOT_INITIALIZED"},
{CKR_USER_TYPE_INVALID, "CKR_USER_TYPE_INVALID"},
{CKR_USER_ANOTHER_ALREADY_LOGGED_IN, "CKR_USER_ANOTHER_ALREADY_LOGGED_IN"},
{CKR_USER_TOO_MANY_TYPES, "CKR_USER_TOO_MANY_TYPES"},
{CKR_WRAPPED_KEY_INVALID, "CKR_WRAPPED_KEY_INVALID"},
{CKR_WRAPPED_KEY_LEN_RANGE, "CKR_WRAPPED_KEY_LEN_RANGE"},
{CKR_WRAPPING_KEY_HANDLE_INVALID, "CKR_WRAPPING_KEY_HANDLE_INVALID"},
{CKR_WRAPPING_KEY_SIZE_RANGE, "CKR_WRAPPING_KEY_SIZE_RANGE"},
{CKR_WRAPPING_KEY_TYPE_INCONSISTENT, "CKR_WRAPPING_KEY_TYPE_INCONSISTENT"},
{CKR_RANDOM_SEED_NOT_SUPPORTED, "CKR_RANDOM_SEED_NOT_SUPPORTED"},
{CKR_RANDOM_NO_RNG, "CKR_RANDOM_NO_RNG"},
{CKR_DOMAIN_PARAMS_INVALID, "CKR_DOMAIN_PARAMS_INVALID"},
{CKR_BUFFER_TOO_SMALL, "CKR_BUFFER_TOO_SMALL"},
{CKR_SAVED_STATE_INVALID, "CKR_SAVED_STATE_INVALID"},
{CKR_INFORMATION_SENSITIVE, "CKR_INFORMATION_SENSITIVE"},
{CKR_STATE_UNSAVEABLE, "CKR_STATE_UNSAVEABLE"},
{CKR_CRYPTOKI_NOT_INITIALIZED, "CKR_CRYPTOKI_NOT_INITIALIZED"},
{CKR_CRYPTOKI_ALREADY_INITIALIZED, "CKR_CRYPTOKI_ALREADY_INITIALIZED"},
{CKR_MUTEX_BAD, "CKR_MUTEX_BAD"},
{CKR_MUTEX_NOT_LOCKED, "CKR_MUTEX_NOT_LOCKED"},
{CKR_FUNCTION_REJECTED, "CKR_FUNCTION_REJECTED"},
{CKR_VENDOR_DEFINED, "CKR_VENDOR_DEFINED"}
};

static const unsigned int pkcs11_error_map_size = sizeof(pkcs11_error_map) / sizeof(pkcs11_error_mapping);

const char *pkcs11_error_message(CK_RV result);

#endif
